/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.InfoBar;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.LayoutPanel;

import java.util.Date;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneHandler;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerHandler;


/**
 * Presenter of the info bar class.
 * 
 * @author Jean-Yves Goubet
 *
 */
public class PInfoBar extends PresenterIF {

  /**
   * @author Mathieu Crassin
   *
   * Instantiate the stub of the orchestra management server
   */
  private final OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);  
  
  /**
   * Information about the user session.
   * 
   * @author Mathieu Crassin
   */
  private String userName = "";
  private String creationDate = "";
  
  private VInfoBarIF view = null;
  
  private HandlerManager eventBus = null;
  
  /**
   * Constructor of the class
   * 
   * @param view : the view of the class
   * 
   * @author Jean-Yves Goubet
   * @param eventBus 
   */
  public PInfoBar(LayoutPanel layoutPanel, HandlerManager eventBus) {
    //
    VInfoBar vInfoBar = new VInfoBar();
    this.bindingView(vInfoBar);
    //
    layoutPanel.add(vInfoBar);
    layoutPanel.setWidgetTopHeight(vInfoBar, 0, Unit.PX, 20, Unit.PX);
    // 
    this.view.getOrchestraLabel().setText("Not connected to Orchestra");
    //
    this.eventBus = eventBus;
    //
    this.initPage();
    //
    this.setDebugID();
    //
    this.addListeners();
  }
  
  protected void bindingView(final Object view) {
    this.view = (VInfoBarIF) view;
  }
  
  /**
   * Listeners on the widgets in the bar
   * 
   * @author Jean-Yves Goubet
   * 
   */
  protected void addListeners() {
    // orchestra
    this.eventBus.addHandler(SuspendTimerEvent.TYPE, new SuspendTimerHandler() {
      public void onSuspend(SuspendTimerEvent suspendTimerEvent) {
        if (suspendTimerEvent.getSuspend()) {
          PInfoBar.this.view.getOrchestraLabel().setText("Not connected to Orchestra");
        }
      }
    });    
    
    this.eventBus.addHandler(RefreshDoneEvent.TYPE, new RefreshDoneHandler() {
      public void onRefreshDone(RefreshDoneEvent refreshDoneEvent) {
        if (PInfoBar.this.view.getOrchestraLabel().getText().equals("Not connected to Orchestra")) {
          PInfoBar.this.view.getOrchestraLabel().setText("Connected to Orchestra");
        }
      }
    });
    
    // refresh listener
    this.view.getRefreshLabel().addMouseOverHandler(new MouseOverHandler() {
      public void onMouseOver(MouseOverEvent arg0) {
        PInfoBar.this.view.getRefreshLabel().setStyleName("disconnectLabelOverStyle");      
      }
    });
    
    this.view.getRefreshLabel().addMouseOutHandler(new MouseOutHandler() {
      public void onMouseOut(MouseOutEvent arg0) {
        PInfoBar.this.view.getRefreshLabel().setStyleName("disconnectLabelStyle");      
      }
    });
    
    // logout listener
    this.view.getLogoutLabel().addClickHandler(new ClickHandler() {
    
      public void onClick(ClickEvent arg0) {
      
        orchestramanagement.sessionLogout(new AsyncCallback<Void>() {
        
          public void onSuccess(Void arg0) {
            Window.Location.reload();
          }
          public void onFailure(Throwable arg0) {
            System.out.println("logout fail !!!!");
          }
        });
      }
    });
  
    // style management for logout 
    this.view.getLogoutLabel().addMouseOverHandler(new MouseOverHandler() {
    
      public void onMouseOver(MouseOverEvent arg0) {
        PInfoBar.this.view.getLogoutLabel().setStyleName("disconnectLabelOverStyle");      
      }
    });
    this.view.getLogoutLabel().addMouseOutHandler(new MouseOutHandler() {
    
      public void onMouseOut(MouseOutEvent arg0) {
        PInfoBar.this.view.getLogoutLabel().setStyleName("disconnectLabelStyle");  
      }
    });
  }
  
  
  /**
   * Get some information to display in the page. 
   * 
   * @author Mathieu Crassin
   */
  private void initPage() {

    // proceed 2 RPC call to get information to display (user name and session creation date)
    
    orchestramanagement.getSessionUserName(new AsyncCallback<String>() {

      public void onFailure(Throwable arg0) {
      }
      
      public void onSuccess(String arg0) {
        
        userName = arg0;
        
        orchestramanagement.getSessionCreationDate(new AsyncCallback<Long>() {

          public void onFailure(Throwable arg0) {
          }
          public void onSuccess(Long arg0) {
            Date date = new Date(arg0);
            DateTimeFormat  processDateFormat = DateTimeFormat.getFormat("dd/MM/yyyy 'at' HH:mm:ss");
            
            creationDate = processDateFormat.format(date);
            
            PInfoBar.this.view.getInfoLabel().setText(userName + " connected since " + creationDate); 
          }
        });         
      }
    });
   
  }
  
  /**
   * Give ids to elements
   * 
   * @author Jean-Yves Goubet
   * 
   */
  protected void setDebugID() {
    this.view.getInfoLabel().ensureDebugId("infoLabelId");
    this.view.getLogoutLabel().ensureDebugId("logoutButtonId");
  }
  
  public VInfoBarIF getView() {
    return this.view;
  }

  public void destroy() {
    // no operations on destroy
    
  }
}
