/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.InstanceDetails;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HorizontalPanel;

import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionClass;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionImage;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IndexFocusPanel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.RowLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.SpecialGrid;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.InstanceGrid;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Presenter of the Instance Details class
 * 
 * @author Jean-Yves Goubet
 *
 */
public class InstanceDetailsGrid extends SpecialGrid {
  
  private OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);
  
  private String instanceId = "";
  
  private PTabLayoutPanel presenterTabLayoutPanel = null;
  
  private int idLimitSize = 10;
  private int nameLimitSize = 10;

  protected HandlerManager eventBus = null;
  
  private static String []columnTitles = {"UUID","Process UUID", "Process Name" ,
    "State","Start Date","Last Update Date","Actions"};
  
  /**
   * Constructor of the presenter
   * 
   * @author Jean-Yves Goubet
   * @param view : the view of the Instance Details class
   * @param instanceId 
   */
  public InstanceDetailsGrid() {
    //
    super(columnTitles, 0, false);
    //
    this.setTitlesDebugId();
  }

  private void setTitlesDebugId() {
    this.getWidget(0, 0).ensureDebugId("instanceIdLabel-UUID-" + instanceId + "-IA");
    this.getWidget(0, 1).ensureDebugId("processIdLabel-UUID-" + instanceId + "-IA");
    this.getWidget(0, 2).ensureDebugId("processNameLabel-UUID-" + instanceId + "-IA");
    this.getWidget(0, 3).ensureDebugId("instanceStateLabel-UUID-" + instanceId + "-IA");
    this.getWidget(0, 4).ensureDebugId("instanceStartDateLabel-UUID-" + instanceId + "-IA");
    this.getWidget(0, 5).ensureDebugId("instanceLastUpdateDateLabel-UUID-" + instanceId + "-IA");
    this.getWidget(0, 6).ensureDebugId("instanceActionsLabel-UUID-" + instanceId + "-IA");
  }
  
  private void setInstanceDebugId() {
    this.getWidget(1, 0).ensureDebugId("instanceIdValue-UUID-" + instanceId + "-IA");
    this.getWidget(1, 1).ensureDebugId("processIdValue-UUID-" + instanceId + "-IA");
    this.getWidget(1, 2).ensureDebugId("processNameValue-UUID-" + instanceId + "-IA");
    this.getWidget(1, 3).ensureDebugId("instanceStateValue-UUID-" + instanceId + "-IA");
    this.getWidget(1, 4).ensureDebugId("instanceStartDateValue-UUID-" + instanceId + "-IA");
    this.getWidget(1, 5).ensureDebugId("instanceLastUpdateDateValue-UUID-" + instanceId + "-IA");
    this.getWidget(1, 6).ensureDebugId("instanceActionsValue-UUID-" + instanceId + "-IA");
    
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(0).getElement(), "activitiesImageID-UUID-" + instanceId + "-IA"); 
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(1).getElement(), "suspendImageID-UUID-" + instanceId + "-IA");
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(2).getElement(), "resumeImageID-UUID-" + instanceId + "-IA");
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(3).getElement(), "exitImageID-UUID-" + instanceId + "-IA");
    ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(1, 6)).getWidget()).
        getWidget(4).getElement(), "removeImageID-UUID-" + instanceId + "-IA");
  }
  
  
  /**
   * This method refresh the info about the instance
   * 
   * @author Jean-Yves Goubet
   */
  public void refreshCurrentList() {
System.out.println("InstanceDetailsGrid - start call");    
    
    orchestramanagement.getInstance(instanceId, new AsyncCallback<MInstance>() {
      public void onSuccess(MInstance instance) {
        // check that the result list contains 1 element   
        InstanceDetailsGrid.this.manageGrid();
        addInstance(instance);
        InstanceDetailsGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }
      
      public void onFailure(Throwable caught) {
        GWT.log("InstanceDetailsGrid - refreshCurrentList : onFailure : " + caught.getMessage());
        InstanceDetailsGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });

  }

  
  /**
   * This method is used to add the instance in the grid.
   * 
   * @author Jean-Yves Goubet
   * @param instance : it's the instance to display on the grid
   * 
   */
  protected void addInstance(final MInstance instance) {
    if (instance != null) {
      // add data to the end of the grid
      int rowIndex = 1;
      //instantiate labels
      RowLabel[] rowLabels = new RowLabel[columnTitles.length - 1];
      rowLabels[0] = formatLabel(instance.getInstanceId(), idLimitSize, rowIndex);
      rowLabels[1] = formatLabel(instance.getProcessId(), idLimitSize, rowIndex);
      rowLabels[2] = formatLabel(instance.getProcessName(), nameLimitSize, rowIndex);
      rowLabels[3] = new RowLabel(instance.getInstanceState(),rowIndex,false);
      rowLabels[4] = new RowLabel(InstanceGrid.dateFormatter(instance.getInstanceStartDate()),rowIndex,false);
      rowLabels[5] = new RowLabel(InstanceGrid.dateFormatter(instance.getInstanceLastUpdateDate()),rowIndex,false);
      //addHandler to labels (for the style) and add labels to grid
      addData(rowLabels);
      
      // add action to instance
      addActionsToInstance(instance.getInstanceState());
      
      // debug id
      this.setInstanceDebugId();
    }
    
    // titles debug id
    this.setTitlesDebugId();
  }  
  
  
  /**
   * Add the different action icon in action column for the instance displayed.
   * The actions are suspend, resume, exit and delete.
   * 
   * @author Jean-Yves Goubet
   * @param state : String that represent the instance state. 
   */
  private void addActionsToInstance(String state) {
    ActionImage [] tab = new ActionImage[5];
    
    
    // activities
    tab[0] = new ActionImage(ImageResources.INSTANCE.activities(), 
        ImageResources.INSTANCE.activitiesDisabled(), 1);
    tab[0].setTitle("activities");
    tab[0].setEnabled(false);
    // suspend action
    tab[1] = new ActionImage(ImageResources.INSTANCE.suspend(), 
        ImageResources.INSTANCE.suspendDisabled(), 1);
    tab[1].setTitle("suspend");
    // resume action 
    tab[2] = new ActionImage(ImageResources.INSTANCE.resume(), 
        ImageResources.INSTANCE.resumeDisabled(), 1);
    tab[2].setTitle("resume");
    // exit action 
    tab[3] = new ActionImage(ImageResources.INSTANCE.exit(), 
        ImageResources.INSTANCE.exitDisabled(), 1);
    tab[3].setTitle("exit");
    // delete action 
    tab[4] = new ActionImage(ImageResources.INSTANCE.deleteIcon(), 
        ImageResources.INSTANCE.deleteIconDisabled(), 1);
    tab[4].setTitle("remove");
    
    this.addActions(tab, state, 6);
  }

  public void setInstanceId(String instanceId) {
    this.instanceId = instanceId;
  }

  public void setPresenterTabLayoutPanel(PTabLayoutPanel presenterTabLayoutPanel) {
    this.presenterTabLayoutPanel = presenterTabLayoutPanel;
  }
  
  
  /**
   * Function call in the SpecialGrid class to attach some actions with each icon in action 
   * column.
   * 
   * @author Mathieu Crassin
   * @param tabImages : a tab that represent the different actions 
   * @param state : state of the information display on the row 
   */
  protected void addClickHandlers(final ActionImage[] tabImages, String state) {
    
    // suspend 
    tabImages[1].setHandler(new ActionClass() {
      public void execute() {
        actionSuspend(tabImages[0]);
      }
    });
    
    // resume
    tabImages[2].setHandler(new ActionClass() {
      public void execute() {
        actionResume(tabImages[0]);
      }
    });

    // exit
    tabImages[3].setHandler(new ActionClass() {
      public void execute() {
        actionExit(tabImages[0]);
      }
    });
    
    // delete
    tabImages[4].setHandler(new ActionClass() {
      public void execute() {
        actionRemove(tabImages[0]);
      }
    });    
    
    // define the state of each action (enable or disable)
    if (state.equals("finished")) {
      tabImages[1].setEnabled(false);
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(false);
      tabImages[4].setEnabled(true);
    } else if (state.equals("running")) {
      tabImages[1].setEnabled(true);
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(true);
      tabImages[4].setEnabled(false);
    } else if (state.equals("suspended")) {
      tabImages[1].setEnabled(false);
      tabImages[2].setEnabled(true);
      tabImages[3].setEnabled(true);
      tabImages[4].setEnabled(false);
    } else if (state.equals("exited")) {
      tabImages[1].setEnabled(false);
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(false);
      tabImages[4].setEnabled(true);
    } else {
      System.out.println("state pb");
    }
  }
  
  
  /**
   * Function call by the handler on the suspend icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */
  protected void actionSuspend(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.suspendInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      public void onSuccess(Void arg0) {
        InstanceDetailsGrid.this.eventBus.fireEvent(
            new ViewRefreshEvent(
                InstanceDetailsGrid.this.instanceId, 
                PTab.Type.LIST_INSTANCE_ACTIVITY, true));
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while suspend instance !", arg0.getMessage());
        }        
      }
    });
  }
  
  
  /**
   * Function call by the handler on the resume icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */
  protected void actionResume(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.resumeInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      public void onSuccess(Void arg0) {
        InstanceDetailsGrid.this.eventBus.fireEvent(
            new ViewRefreshEvent(
                InstanceDetailsGrid.this.instanceId, 
                PTab.Type.LIST_INSTANCE_ACTIVITY, true));
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while resume instance !", arg0.getMessage());
        }           
      }
    });
  }
  
  /**
   * Function call by the handler on the exit icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */  
  protected void actionExit(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.exitInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      
      public void onSuccess(Void arg0) {
        InstanceDetailsGrid.this.eventBus.fireEvent(
            new ViewRefreshEvent(InstanceDetailsGrid.this.instanceId, PTab.Type.LIST_INSTANCE_ACTIVITY, true));
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while exit instance !", arg0.getMessage());
        }   
      }
    });
  }
  
  /**
   * Function call by the handler on the delete icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */  
  protected void actionRemove(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.removeInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      
      public void onSuccess(Void arg0) {
        // delete instance activity tab if exists
        InstanceDetailsGrid.this.presenterTabLayoutPanel.removeTab(instanceId, PTab.Type.LIST_INSTANCE_ACTIVITY);
        InstanceDetailsGrid.this.eventBus.fireEvent(
            new ViewRefreshEvent(InstanceDetailsGrid.this.instanceId, PTab.Type.LIST_INSTANCE_ACTIVITY, true));
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while delete instance !", arg0.getMessage());
        }   
      }
    });
  }  
  
  
  public void setHandlerManager(final HandlerManager eventBus) {
    this.eventBus  = eventBus;
  }

  /**
   * This function is empty because the sort functionality is inactive on this grid.
   * But we have to implement it because this function belong to the SpecialGrid class. 
   * 
   * @author Mathieu Crassin
   */  
  public void sortAndDisplayList() {
  }
}
